/**
 * Une liste de rsultats ordonns (selon {compareTo}), reprsentant le
 * classement d'une course cycliste.  
 * 
 * @author Benoit Donnet et Charles Pecheur, UCLouvain
 * @version Septembre 2013
 */
public interface Classement {

	/**
	 * Ajoute un rsultat dans le classement.
	 * 
	 * @pre r != null
	 * @post le rsultat {r} a t insr selon l'ordre du classement. En cas
	 *       d'ex-aequo, {r} est insr aprs les autres rsultats de mme
	 *       ordre.
	 */
	public void add(Resultat r);

	/**
	 * Retire un rsultat du classement.
	 * 
	 * @pre c != null
	 * @post retire le premier (meilleur) rsultat pour le coureur {c} du classement. {c}
	 *       est compar au sens de {equals}. Retourne {true} si un rsultat a t retir,
	 *       {false} si {c} n'est pas trouv dans la liste.
	 */
	public boolean remove(Coureur c);

	/**
	 * Taille du classement.
	 * 
	 * @pre -
	 * @post retourne le nombre de rsultats dans le classement.
	 */
	public int size();

	/**
	 * Le rsultat d'un coureur.
	 * 
	 * @pre c != null
	 * @post retourne le premier (meilleur) rsultat du coureur {c} dans le
	 *       classement. Retourne {null} si le coureur ne figure pas dans le
	 *       classement.
	 */
	public Resultat get(Coureur c);
	
	/**
	 * La position d'un coureur dans le classement.
	 * 
	 * @pre c != null
	 * @post retourne la position du coureur {c} dans le classement,  partir de
	 *       1 pour la tte de classement. Si le coureur figure plusieurs fois
	 *       dans le classement, la premire (meilleure) position est retourne.
	 *       Retourne -1 si le coureur ne figure pas dans le classement.
	 */
	public int getPosition(Coureur c);

	/**
	 * Le classement sous forme de texte.
	 * 
	 * @pre -
	 * @post retourne le classement sous forme de texte, avec une ligne par
	 *       rsultat.
	 */
	public String toString();
}
